@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2020, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -29,15 +29,19 @@
 """Unittests for mysql.connector.connection
 """
 
+import copy
 import os
 import logging
+import platform
 import timeit
 import unittest
 from decimal import Decimal
 import io
 import socket
+import sys
 
 import tests
+from . import check_tls_versions_support
 from . import PY2
 
 try:
@@ -49,8 +53,13 @@ except ImportError:
 
 from mysql.connector.conversion import (MySQLConverterBase, MySQLConverter)
 from mysql.connector import (connect, connection, network, errors,
-                             constants, cursor, abstracts, catch23)
+                             constants, cursor, abstracts, catch23,
+                             HAVE_DNSPYTHON)
+from mysql.connector.errors import InterfaceError
 from mysql.connector.optionfiles import read_option_files
+from mysql.connector.network import TLS_V1_3_SUPPORTED
+from mysql.connector.utils import linux_distribution
+from mysql.connector.version import VERSION, LICENSE
 
 LOGGER = logging.getLogger(tests.LOGGER_NAME)
 
@@ -1218,7 +1227,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         conn_id = cnx.connection_id
         cnx.reconnect(attempts=5)
         exp = conn_id + 1
-        self.assertEqual(exp, cnx.connection_id)
+        self.assertGreaterEqual(cnx.connection_id, exp)
 
     def test_ping(self):
         """Ping the MySQL server"""
@@ -1707,6 +1716,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
             else:
                 self.assertNotEqual((b'2',), self.cnx.get_rows()[0][0])
 
+    @unittest.skipIf(os.environ.get("PB2WORKDIR"), "Do not run on PB2")
     @unittest.skipIf(tests.MYSQL_VERSION <= (5, 7, 1), "Shutdown CMD "
                      "not tested with MySQL version 5.6 (BugOra17422299)")
     def test_shutdown(self):
@@ -1788,3 +1798,518 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         self.assertEqual(exp, cur.fetchall())
         cur.execute("DROP TABLE IF EXISTS local_data")
         cur.close()
+
+    @tests.foreach_cnx()
+    def test_connection_attributes_defaults(self):
+        """Test default connection attributes"""
+        if os.name == "nt":
+            if "64" in platform.architecture()[0]:
+                platform_arch = "x86_64"
+            elif "32" in platform.architecture()[0]:
+                platform_arch = "i386"
+            else:
+                platform_arch = platform.architecture()
+            os_ver = "Windows-{}".format(platform.win32_ver()[1])
+        else:
+            platform_arch = platform.machine()
+            if platform.system() == "Darwin":
+                os_ver = "{}-{}".format("macOS", platform.mac_ver()[0])
+            else:
+                os_ver = "-".join(linux_distribution()[0:2])
+
+        license_chunks = LICENSE.split(" ")
+        if license_chunks[0] == "GPLv2":
+            client_license = "GPL-2.0"
+        else:
+            client_license = "Commercial"
+
+        cur = self.cnx.cursor()
+        # Verify user defined session-connection-attributes are in the server
+        cur.execute("SHOW VARIABLES LIKE \"pseudo_thread_id\"")
+        pseudo_thread_id = cur.fetchall()[0][1]
+        get_attrs = ("SELECT ATTR_NAME, ATTR_VALUE FROM "
+                    "performance_schema.session_account_connect_attrs "
+                    "where PROCESSLIST_ID = \"{}\"")
+        cur.execute(get_attrs.format(pseudo_thread_id))
+        rows = cur.fetchall()
+        res_dict = dict(rows)
+        if CMySQLConnection is not None and \
+           isinstance(self.cnx, CMySQLConnection):
+            expected_attrs = {
+                "_source_host": socket.gethostname(),
+                "_connector_name": "mysql-connector-python",
+                "_connector_license": client_license,
+                "_connector_version": ".".join([str(x) for x in VERSION[0:3]]),
+            }
+        else:
+            expected_attrs = {
+                "_pid": str(os.getpid()),
+                "_platform": platform_arch,
+                "_source_host": socket.gethostname(),
+                "_client_name": "mysql-connector-python",
+                "_client_license": client_license,
+                "_client_version": ".".join([str(x) for x in VERSION[0:3]]),
+                "_os": os_ver
+            }
+        # Note that for an empty string "" value the server stores a Null value
+        for attr_name in expected_attrs:
+            self.assertEqual(expected_attrs[attr_name], res_dict[attr_name],
+                             "Attribute {} with value {} differs of {}"
+                             "".format(attr_name, res_dict[attr_name],
+                                       expected_attrs[attr_name]))
+
+    @unittest.skipIf(sys.version_info < (2, 7, 9), "The support for SSL is "
+                     "not available for Python versions < 2.7.9.")
+    @unittest.skipIf(tests.MYSQL_VERSION < (5, 7, 40), "TLSv1.1 incompatible")
+    @tests.foreach_cnx()
+    def test_get_connection_with_tls_version(self):
+        if isinstance(self.cnx, connection.MySQLConnection):
+            connector_class = connection.MySQLConnection
+        else:
+            connector_class = CMySQLConnection
+
+        # Test None value is returned if no schema name is specified
+        settings = tests.get_mysql_config()
+        if "unix_socket" in settings:
+            settings.pop("unix_socket")
+
+        # tls_versions and ssl_disabled=True
+        settings["tls_versions"] = ["TLSv1"]
+        settings["ssl_disabled"] = True
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("along with ssl_disabled" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # tls_versions and ssl_disabled=True
+        settings.pop("tls_versions")
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["ssl_disabled"] = True
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("along with ssl_disabled" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Empty tls_version list
+        settings.pop("ssl_disabled")
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["tls_versions"] = []
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("At least one" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Empty tls_ciphersuites list using dict settings
+        settings["tls_ciphersuites"] = []
+        settings["tls_versions"] = ["TLSv1"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("No valid cipher suite" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Empty tls_ciphersuites list without tls-versions
+        settings["tls_ciphersuites"] = []
+        settings.pop("tls_versions")
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("No valid cipher suite" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Given tls-version not in ["TLSv1.1", "TLSv1.2", "TLSv1.3"]
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["tls_versions"] = ["TLSv0.2", "TLSv1.7", "TLSv10.2"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+
+        # Repeated values in tls_versions on dict settings
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["tls_versions"] = ["TLSv1.2", "TLSv1.1", "TLSv1.2"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+
+        # Empty tls_versions on dict settings
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["tls_versions"] = []
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("At least one TLS" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Verify unkown cipher suite case?
+        settings["tls_ciphersuites"] = ["NOT-KNOWN"]
+        settings["tls_versions"] = ["TLSv1.2"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+
+        # Verify AttributeError exception is raised With invalid TLS version
+        settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+        settings["tls_versions"] = ["TLSv8"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+        self.assertTrue(("not recognized" in str(context.exception)),
+                        "Unexpected exception message found: {}"
+                        "".format(context.exception))
+
+        # Verify unkown cipher suite case?
+        settings["tls_ciphersuites"] = ["NOT-KNOWN"]
+        settings["tls_versions"] = ["TLSv1.2"]
+        with self.assertRaises(AttributeError) as context:
+            _ = connector_class(**settings)
+
+        # Verify unsupported TLSv1.3 version is accepted (connection success)
+        settings["tls_ciphersuites"] = None
+        settings["tls_versions"] = ["TLSv1.3", "TLSv1.2"]
+        # Connection must be successfully
+        _ = connector_class(**settings)
+
+        err_msg = ("Not using the expected TLS version: {}, instead the "
+                   "connection used: {}.")
+
+        supported_tls = check_tls_versions_support(
+            ["TLSv1.2", "TLSv1.1", "TLSv1"])
+        if not supported_tls:
+            self.fail("No TLS version to test: {}".format(supported_tls))
+
+        # Following tests requires TLSv1.2
+        if tests.MYSQL_VERSION < (8, 0, 17):
+            return
+
+        if len(supported_tls) > 1:
+            # Verify given TLS version is used
+            settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+            for test_ver in supported_tls:
+                expected_ssl_version = test_ver
+                settings["tls_versions"] = [test_ver]
+                cnx = connector_class(**settings)
+                cur = cnx.cursor()
+                cur.execute("SHOW STATUS LIKE 'Ssl_version%'")
+                res = cur.fetchall()
+    
+                self.assertEqual(res[0][1], expected_ssl_version,
+                                 err_msg.format(expected_ssl_version, res))
+
+        # Verify the newest TLS version is used from the given list
+        exp_res = ["TLSv1.2", "TLSv1.2", "TLSv1.2"]
+        test_vers = [["TLSv1", "TLSv1.2", "TLSv1.1"], ["TLSv1.1", "TLSv1.2"],
+                     ["TLSv1.2", "TLSv1.1"]]
+        for test_ver, exp_ver in zip(test_vers, exp_res):
+            settings["tls_versions"] = test_ver
+            cnx = connector_class(**settings)
+            cur = cnx.cursor()
+            res = cur.execute("SHOW STATUS LIKE 'ssl_version%'")
+            res = cur.fetchall()
+            self.assertEqual(res[0][1], exp_ver, "On test case {}, {}"
+                             "".format(test_ver, err_msg.format(exp_ver, res)))
+
+        # Verify given TLS cipher suite is used
+        exp_res = ["DHE-RSA-AES128-GCM-SHA256", "DHE-RSA-AES128-GCM-SHA256",
+                   "DHE-RSA-AES128-GCM-SHA256"]
+        test_ciphers = [["DHE-RSA-AES128-GCM-SHA256"],
+                        ["DHE-RSA-AES128-GCM-SHA256"],
+                        ["TLS_DHE_RSA_WITH_AES_128_GCM_SHA256"]]
+        settings["tls_versions"] = ["TLSv1.2"]
+        for test_cipher, exp_ver in zip(test_ciphers, exp_res):
+            settings["tls_ciphersuites"] = test_cipher
+            cnx = connector_class(**settings)
+            cur = cnx.cursor()
+            cur.execute("SHOW STATUS LIKE 'Ssl_cipher%'")
+            res = cur.fetchall()
+            self.assertEqual(res[0][1], exp_ver, "Unexpected TLS version found:"
+                             " {} for: {}".format(res[0][1], test_cipher))
+
+        # Verify one of TLS cipher suite is used from the given list
+        exp_res = ["DHE-RSA-AES256-SHA256", "DHE-RSA-AES256-SHA256",
+                   "DHE-RSA-AES128-GCM-SHA256"]
+        test_ciphers = ["TLS_DHE_RSA_WITH_AES_256_CBC_SHA256",
+                        "DHE-RSA-AES256-SHA256",
+                        "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256"]
+        settings["tls_ciphersuites"] = test_ciphers
+        cnx = connector_class(**settings)
+        cur = cnx.cursor()
+        cur.execute("SHOW STATUS LIKE 'Ssl_cipher%'")
+        res = cur.fetchall()
+        self.assertIn(res[0][1], exp_res, "Unexpected TLS version found"
+                      ": {} not in {}".format(res[0][1], exp_res))
+
+        if "TLSv1.1" in supported_tls:
+            # Verify connection success with either TLS given version when.
+            # TLSv1.3 is not supported.
+            settings["tls_ciphersuites"] = ["DHE-RSA-AES256-SHA"]
+            settings["tls_versions"] = ["TLSv1.3", "TLSv1.1"]
+
+            cnx = connector_class(**settings)
+            cur = cnx.cursor()
+            cur.execute("SHOW STATUS LIKE 'ssl_version%'")
+            res = cur.fetchall()
+            self.assertNotEqual(res[0][1], 'TLSv1.2', "Unexpected TLS version "
+                                "found: {}".format(res[0][1]))
+
+        # Verify error when TLSv1.3 is not supported.
+        if not TLS_V1_3_SUPPORTED:
+            settings["tls_versions"] = ["TLSv1.3"]
+            with self.assertRaises(AttributeError) as context:
+                _ = connector_class(**settings)
+
+    def test_connection_attributes_user_defined(self):
+        """Tests defined connection attributes"""
+        config = tests.get_mysql_config()
+        use_pure_values = [True]
+        if HAVE_CMYSQL:
+            use_pure_values.append(False)
+        for use_pure in use_pure_values:
+            config['use_pure'] = use_pure
+
+            # Validate an error is raised if user defined connection attributes
+            # are invalid
+            invalid_conn_attrs = [{1:"1"}, {1:2}, {"_invalid":""}, {"_": ""},
+                                  123, 123.45, "text", {"_invalid"}, ['_a1=2',]]
+
+            for invalid_attr in invalid_conn_attrs:
+                test_config = copy.deepcopy(config)
+                test_config['conn_attrs'] = invalid_attr
+                with self.assertRaises(errors.InterfaceError) as _:
+                    _ = connect(**test_config)
+                    LOGGER.error("InterfaceError not raised while testing "
+                                 "invalid attribute: {}".format(invalid_attr))
+
+            # Test error is raised for attribute name starting with '_'
+            connection_attributes = [
+                {"foo": "bar", "_baz": "zoom"},
+                {"_baz": "zoom"},
+                {"foo": "bar", "_baz": "zoom", "puuuuum": "kaplot"}
+            ]
+            for invalid_attr in connection_attributes:
+                test_config = copy.deepcopy(config)
+                test_config['conn_attrs'] = invalid_attr
+                with self.assertRaises(errors.InterfaceError) as _:
+                    _ = connect(**test_config)
+                    LOGGER.error("InterfaceError not raised while testing "
+                                 "invalid attribute: {}".format(invalid_attr))
+
+            # Test error is raised for attribute name size exceeds 32 characters
+            connection_attributes = [
+                {"foo": "bar", "p{}w".format("o"*31): "kaplot"},
+                {"p{}w".format("o"*31): "kaplot"},
+                {"baz": "zoom", "p{}w".format("o"*31): "kaplot", "a": "b"}
+            ]
+            for invalid_attr in connection_attributes:
+                test_config = copy.deepcopy(config)
+                test_config['conn_attrs'] = invalid_attr
+                with self.assertRaises(errors.InterfaceError) as context:
+                    _ = connect(**test_config)
+                    LOGGER.error("InterfaceError not raised while testing "
+                                 "invalid attribute: {}".format(invalid_attr))
+
+                self.assertTrue("exceeds 32 characters limit size" in
+                                context.exception.msg)
+
+            # Test error is raised for attribute value size exceeds 1024 characters
+            connection_attributes = [
+                {"foo": "bar", "pum": "kr{}nk".format("u"*1024)},
+                {"pum": "kr{}nk".format("u"*1024)},
+                {"baz": "zoom", "pum": "kr{}nk".format("u"*1024), "a": "b"}
+            ]
+            for invalid_attr in connection_attributes:
+                test_config = copy.deepcopy(config)
+                test_config['conn_attrs'] = invalid_attr
+                with self.assertRaises(errors.InterfaceError) as context:
+                    _ = connect(**test_config)
+                    LOGGER.error("InterfaceError not raised while testing "
+                                 "invalid attribute: {}".format(invalid_attr))
+
+            self.assertTrue("exceeds 1024 characters limit size" in
+                            context.exception.msg)
+
+            # Validate the user defined attributes are created in the server
+            connection_attributes = {
+                "foo": "bar",
+                "baz": "zoom",
+                "quash": "",
+                "puuuuum": "kaplot"
+            }
+            test_config = copy.deepcopy(config)
+            test_config['conn_attrs'] = connection_attributes
+            cnx = connect(**test_config)
+            cur = cnx.cursor()
+            cur.execute("SHOW VARIABLES LIKE \"pseudo_thread_id\"")
+            pseudo_thread_id = cur.fetchall()[0][1]
+            get_attrs = ("SELECT ATTR_NAME, ATTR_VALUE FROM "
+                        "performance_schema.session_account_connect_attrs "
+                        "where PROCESSLIST_ID = \"{}\"")
+            cur.execute(get_attrs.format(pseudo_thread_id))
+            rows = cur.fetchall()
+            res_dict = dict(rows)
+            expected_attrs = copy.deepcopy(connection_attributes)
+            # Note: For an empty string "" value the server stores a Null value
+            expected_attrs["quash"] = None
+            for attr_name in expected_attrs:
+                self.assertEqual(expected_attrs[attr_name], res_dict[attr_name],
+                                 "Attribute {} with value {} differs of {}"
+                                 "".format(attr_name, res_dict[attr_name],
+                                           expected_attrs[attr_name]))
+
+    @unittest.skipIf(tests.MYSQL_VERSION < (8, 0, 19),
+                     "MySQL 8.0.19+ is required for DNS SRV")
+    @unittest.skipIf(not HAVE_DNSPYTHON,
+                     "dnspython module is required for DNS SRV")
+    def test_dns_srv(self):
+        config = tests.get_mysql_config().copy()
+        config.pop("unix_socket", None)
+        config.pop("port", None)
+
+        # The value of 'dns-srv' must be a boolean
+        config["dns_srv"] = "true"
+        self.assertRaises(InterfaceError, connect, **config)
+        config["dns_srv"] = "false"
+        self.assertRaises(InterfaceError, connect, **config)
+        config["dns_srv"] = 0
+        self.assertRaises(InterfaceError, connect, **config)
+        config["dns_srv"] = 1
+        self.assertRaises(InterfaceError, connect, **config)
+        config["dns_srv"] = True
+
+        # Specifying a port number with DNS SRV lookup is not allowed
+        config["port"] = 3306
+        self.assertRaises(InterfaceError, connect, **config)
+        del config["port"]
+
+        # Using Unix domain sockets with DNS SRV lookup is not allowed
+        config["unix_socket"] = "/tmp/mysql.sock"
+        self.assertRaises(InterfaceError, connect, **config)
+        del config["unix_socket"]
+
+
+class WL13335(tests.MySQLConnectorTests):
+    """WL#13335: Avoid set config values whit flag CAN_HANDLE_EXPIRED_PASSWORDS
+    """
+    def setUp(self):
+        self.config = tests.get_mysql_config()
+        cnx = connection.MySQLConnection(**self.config)
+        self.user = "expired_pw_user"
+        self.passw = "i+QEqGkFr8h5"
+        self.hosts = ["localhost", "127.0.0.1"]
+        for host in self.hosts:
+            cnx.cmd_query("CREATE USER '{}'@'{}' IDENTIFIED BY "
+                          "'{}'".format(self.user, host, self.passw))
+            cnx.cmd_query("GRANT ALL ON *.* TO '{}'@'{}'"
+                          "".format(self.user, host))
+            cnx.cmd_query("ALTER USER '{}'@'{}' PASSWORD EXPIRE"
+                          "".format(self.user, host))
+        cnx.close()
+
+    def tearDown(self):
+        cnx = connection.MySQLConnection(**self.config)
+        for host in self.hosts:
+            cnx.cmd_query("DROP USER '{}'@'{}'".format(self.user, host))
+        cnx.close()
+
+    @tests.foreach_cnx()
+    def test_connect_with_can_handle_expired_pw_flag(self):
+        cnx_config = self.config.copy()
+        cnx_config['user'] = self.user
+        cnx_config['password'] = self.passw
+        flags = constants.ClientFlag.get_default()
+        flags |= constants.ClientFlag.CAN_HANDLE_EXPIRED_PASSWORDS
+        cnx_config['client_flags'] =flags
+        # connection must be successful
+        _ = self.cnx.__class__(**cnx_config)
+
+
+class WL13334(tests.MySQLConnectorTests):
+    """WL#13334: Failover and multihost
+    """
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
+    def test_failover_init(self):
+        cnx_config = tests.get_mysql_config()
+        user = cnx_config['user']
+        passw = cnx_config['password']
+        host = cnx_config['host']
+        port = cnx_config['port']
+
+        # check invalid priority values
+        settings_org = {
+            "failover": [
+                {"host":"127.0.1.2", "port":port, "user": user,
+                 "priority":80},
+                {"host":"127.0.1.5", "port":port, "user": user,
+                 "priority":80},
+                {"host":"127.0.1.7", "port":port, "user": user,
+                 "priority":10},
+                {"host":"127.0.1.10", "port":port, "user": user,
+                 "priority":10}],
+            "user": "root",
+            "passwd": passw,
+        }
+
+        # check invalid priority values: negative
+        settings = settings_org.copy()
+        settings["failover"].append({"host":host, "port":port, "user": user,
+                 "priority":-1})
+        with self.assertRaises(InterfaceError):
+            _ = connect(**settings)
+
+        # check invalid priority values: out of range
+        settings = settings_org.copy()
+        settings["failover"].append({"host":host, "port":port, "user": user,
+                 "priority":101})
+        with self.assertRaises(InterfaceError):
+            _ = connect(**settings)
+
+        # check invalid priority values: not int type
+        settings = settings_org.copy()
+        settings["failover"].append({"host":host, "port":port, "user": user,
+                 "priority":"A"})
+        with self.assertRaises(InterfaceError):
+            _ = connect(**settings)
+
+        # check all servers has priority or none error
+        settings = settings_org.copy()
+        settings["failover"].append({"host":host, "port":port, "user": user})
+        with self.assertRaises(InterfaceError):
+            _ = connect(**settings)
+
+    def test_failover(self):
+        cnx_config = tests.get_mysql_config()
+        user = cnx_config['user']
+        passw = cnx_config['password']
+        host = cnx_config['host']
+        port = cnx_config['port']
+
+        settings = {
+            "failover": [
+                {"host":"127.0.1.2", "port":port, "user": user,
+                 "priority":80},
+                {"host":"127.0.1.5", "port":port, "user": user,
+                 "priority":80},
+                {"host":"127.0.1.7", "port":port, "user": user,
+                 "priority":10},
+                {"host":"127.0.1.10", "port":port, "user": user,
+                 "priority":10}],
+            "user": "root",
+            "passwd": passw,
+        }
+
+        # connection must fail with error: "Unable to connect to any of the target hosts"
+        with self.assertRaises(InterfaceError) as context:
+            _ = connect(**settings)
+
+        self.assertTrue(
+            ("Unable to connect to any of the target hosts" in context.exception.msg),
+            "Unexpected exception message found: {}".format(context.exception.msg))
+
+        # Verify connection is successful
+        settings["failover"].append({"host":host, "port":port, "user": user,
+                 "priority":100})
+        # connection must be successful
+        _ = connect(**cnx_config)
